<?php

use WHMCS\View\Menu\Item as MenuItem;
use WHMCS\Database\Capsule;

define('ED_IDT','Identification Document Type');
define('ED_ID','ID Document');

function getEuroDNSConfig()
{
    require_once dirname(__FILE__).'/classEuroDNS.php';

    try {
        $data = Capsule::table('tblregistrars')
            ->where('registrar', 'EuroDNS')
            ->where('setting', 'Username')
            ->first();
        $username = decrypt($data->value);

        $data = Capsule::table('tblregistrars')
            ->where('registrar', 'EuroDNS')
            ->where('setting', 'Password')
            ->first();
        $password = decrypt($data->value);

        $data = Capsule::table('tblregistrars')
            ->where('registrar', 'EuroDNS')
            ->where('setting', 'TestMode')
            ->first();
        $testMode = decrypt($data->value);

        return [
            'Username' => $username,
            'Password' => $password,
            'TestMode' => $testMode,
        ];
    } catch (Exception $exc) {
        logModuleCall('EuroDNS', 'getEuroDNSConfig', null, $exc->getMessage(), null, null);
        return []; // return an empty array if an error occurs
    }
}

function eurodns_ShoppingCartValidateDomainsConfig($vars)
{
    try {
        $attr = getEuroDNSConfig();
        $API = new EuroDNS($attr);

        foreach ($_SESSION['cart']['domains'] as $key => $domain) {
            $parts = explode('.', $domain['domain']);
            $tld = end($parts);
            unset($parts[key($parts)]);

            $params = [
                'sld' => implode('.', $parts),
                'tld' => $tld,
            ];
            $result = $API->check($params);
        }

        if (!$result) {
            $values['error'] = $API->getError();
        }

        return $values;
    } catch (Exception $e) {
        return ['error' => $e->getMessage()];
    }
}

function GetDomainHelper($uID,$dID)
{
    $command = 'GetClientsDomains';
    $postData = [
        'clientid' => $uID,
        'stats' => true,
    ];

    $results = localAPI($command, $postData);

    foreach ($results['domains']['domain'] as $key => $domain) {
        if ($domain['id'] === $dID) {
            return $domain;
        }
    }

    return null;
}

function IsRegistrarEurodns($uID,$dID)
{
    $domain = GetDomainHelper($uID,$dID);

    if ($domain && 'EuroDNS' === $domain['registrar'] && class_exists('EuroDNS')) {
        return $domain;
    }

    return false;
}

function GetAdditionalFieldFromDB($dID,$fieldName)
{
    $requestId = Capsule::table('tbldomainsadditionalfields')
        ->where('domainid', $dID)
        ->where('name', $fieldName)
        ->get();

    if ($requestId->isEmpty()) {
        return null;
    }

    return $requestId;
}

function InsertAdditionalFieldInDB($dID,$fieldName,$fieldValue)
{
    if ($dID && $fieldName && $fieldValue) {
        try {
            Capsule::table('tbldomainsadditionalfields')->insert(
                [
                    'domainid' => $dID,
                    'name' => $fieldName,
                    'value' => $fieldValue,
                ]
            );
            return true;
        } catch (Exception $exc) {
            // Log the error in case of an exception
            logModuleCall('EuroDNS', 'InsertAdditionalFieldInDB', compact('dID', 'fieldName', 'fieldValue'), $exc->getMessage(), null, null);
            return false;
        }
    }
    return null;
}

function UpdateAdditionalFieldInDB($dID,$fieldName,$fieldValue)
{
    if ($dID && $fieldName && $fieldValue) {
        try {
            Capsule::table('tbldomainsadditionalfields')
                ->where('domainid', $dID)
                ->where('name', $fieldName)
                ->update(
                    ['value' => $fieldValue]
                );
            return true;
        } catch (Exception $exc) {
            // Log the error in case of an exception
            logModuleCall('EuroDNS', 'UpdateAdditionalFieldInDB', compact('dID', 'fieldName', 'fieldValue'), $exc->getMessage(), null, null);
            return false;
        }
    }
    return null;
}

add_hook('ShoppingCartValidateDomainsConfig', 1, 'eurodns_ShoppingCartValidateDomainsConfig');
add_hook('ClientAreaPageDomainDNSManagement', 10, function ($vars) {

    if (IsRegistrarEurodns($vars['userid'],$vars['domainid'])) 
    {
        $formUrl = $_SERVER['PHP_SELF'].'?action=domaindns';
        $DID = end(explode('=', $vars['breadcrumb'][3]['link']));
        $token = $vars['token'];
        $parts = explode('.', $vars['breadcrumb'][3]['label']);
        $tld = end($parts);
        unset($parts[key($parts)]);
        $API = new EuroDNS(getEuroDNSConfig());
        $res = $API->zoneInfo(['sld' => implode('.', $parts), 'tld' => $tld]);

        if($res)
        {
            echo $API->GetDNSForm($res, $formUrl, $token, $DID);
        }
    }
});

add_hook('ClientAreaHeaderOutput', 1, function ($vars) {

    if (empty($vars['domain']) || (array_key_exists('loggedin',$vars) && !$vars['loggedin'] )) {
        return '';
    }

    $domain = $vars['domain'];
    $parts = explode('.', $domain);
    
    if (count($parts) < 2 || empty($parts[0]) || empty($parts[1])) {
        logModuleCall('EuroDNS', 'ClientAreaHeaderOutput', 'Invalid domain format: ' . $domain, '', null, null);
        return '';
    }

    $tld = substr($domain, strpos($domain, '.') + 1);
    $data = getEuroDNSConfig();
    $API = new EuroDNS($data);
    $result = $API->getInfo(['sld' => $parts[0], 'tld' => $tld]);

    if(!$result)
    {
        logModuleCall('EuroDNS', 'ClientAreaHeaderOutput:getInfo', 'Could not load Domain information for: ' . $domain, '', null, null);
        return '';
    }

    if (in_array(end(explode('.', $tld)), $API->GetNoLockForTLD())) {
        $addStyle = '<script>jQuery(function(){jQuery("a[href=\"#tabReglock\"]").parent().hide()});</script>';
    }

    // Check for pending messages and output necessary scripts
    if (!empty($API->getPendingMessage())) {
        return '<style>
            #Primary_Sidebar-Domain_Details_Management-Auto_Renew_Settings, ul.dropdown-menu.text-left.dropdown-menu-right li:nth-child(3) {
                display:none !important;
            }
        </style>'.$addStyle.$API->GetInfoMessageFrontend('Pending Action: '.$domain, 
        'There are some pending actions from your registrar. The information that appears on the page might be not up to date. Current action: <strong>"'.$API->getPendingMessage().'"</strong>. Some actions like changing nameservers will not be performed.');
    }

    return '<style>
        #Primary_Sidebar-Domain_Details_Management-Auto_Renew_Settings, ul.dropdown-menu.text-left.dropdown-menu-right li:nth-child(3) {
            display:none !important;
        }
    </style>'.$addStyle;
});

add_hook('DomainEdit', 1, function ($vars) {
    if ($domain = IsRegistrarEurodns($vars['userid'], $vars['domainid'])) {
        if (empty($domain['domainname']) || strpos($domain['domainname'], '.') === false) {
            logModuleCall('EuroDNS', 'DomainEdit', "Invalid domain name: {$domain['domainname']}", '', null, null);
            return;
        }

        $array = explode('.', $domain['domainname'], 2);
        if (count($array) < 2 || empty($array[0]) || empty($array[1])) {
            logModuleCall('EuroDNS', 'DomainEdit', "Malformed domain name: {$domain['domainname']}", '', null, null);
            return;
        }

        $data = getEuroDNSConfig();
        $data['sld'] = $array[0];
        $data['tld'] = $array[1];

        $API = new EuroDNS($data);

        try {
            $info = $API->getInfo($data);
            
            if ($info) {
                $info = new SimpleXMLElement($info);
                $autoRenew = strtolower((string) $info->xpath('/response/resData/domain:renewal')[0]);
                $whmcsRenew = (string) $domain['donotrenew'];

                if (('autoexpire' === $autoRenew && '0' === $whmcsRenew) || ('autorenew' === $autoRenew && '1' === $whmcsRenew)) {
                    $data['renewal'] = ('1' === $whmcsRenew ? 'autoexpire' : 'autorenew');
                    $API->SetRenewalMode($data);
                    logModuleCall('EuroDNS', 'DomainEdit:SetRenewalMode', $API->getRequest(), $API->getResponse(), null, null);
                }
            }
        } catch (Exception $e) {
            logModuleCall('EuroDNS', 'DomainEdit:Exception', compact('data'), $e->getMessage(), null, null);
        }
    }
});

add_hook('ClientAreaDomainDetailsOutput', 1, function ($vars) {
    
    if(IsRegistrarEurodns($vars['userid'],$vars['domainid']))
    {
        $domain = $vars['domain'];
        $data = getEuroDNSConfig();
        $mydomain = $domain->domain;
        $tld = substr($mydomain, strpos($mydomain, '.') + 1);
        $API = new EuroDNS($data);

        if (!$mydomain || empty($tld)) {
            return;
        }

        $tldNeed = $API->getCachedInfo(['tld', $tld, 'lockingEnabled'], true);

        if (!$tldNeed && empty($tldNeed)) {
            $infoTLD = $API->infoTld($tld);

            if($infoTLD)
            {
                $tldNeed = $infoTLD['lockingEnabled'];
                if ('no' === strtolower($tldNeed)) {
                    return '<style>
                        .alert.alert-danger, #tabOverview a[href="#tabReglock"]
                        {
                            display:none !important;
                        }
                    </style>';
                }
            }
        }
    }
});

add_hook('ClientAreaPrimarySidebar', 1, function (MenuItem $primarySidebar) {
    $data = getEuroDNSConfig();
    $domain = Menu::context('domain');
    $mydomain = $domain->domain;
    $tld = substr($mydomain, strpos($mydomain, '.') + 1);

    if (!$mydomain || empty($tld)) {
        return;
    }

    $API = new EuroDNS($data);
    $tldNeed = $API->getCachedInfo(['tld', $tld, 'lockingEnabled'], true);

    if (!$tldNeed && empty($tldNeed)) {
        $infoTLD = $API->infoTld($tld);

        if($infoTLD)
        {
            $tldNeed = $infoTLD['lockingEnabled'];

            if ('no' === strtolower($tldNeed)) {
                if (!is_null($primarySidebar->getChild('Domain Details Management'))) {
                    $primarySidebar->getChild('Domain Details Management')
                        ->removeChild('Registrar Lock Status')
                    ;
                }
            }
        }
    }
});

add_hook('AdminClientDomainsTabFields', 1, function($vars) {

    if((array_key_exists('userid',$_GET) && array_key_exists('id',$vars)) && ($domain = IsRegistrarEurodns($_GET['userid'],$vars['id'])))
    {
        $pos = strpos(strtolower($domain['domainname']), '.au');
        if($pos)
        {

            $addFieldT = GetAdditionalFieldFromDB($vars['id'],ED_IDT);
            $addFieldID = GetAdditionalFieldFromDB($vars['id'],ED_ID);

            if(($addFieldT instanceof Illuminate\Support\Collection) && $addFieldT->count()>0)
            {
                $addFieldT = $addFieldT->first()->value;
            }

            if(($addFieldID instanceof Illuminate\Support\Collection) && $addFieldID->count()>0)
            {
                $protocol = $_SERVER['HTTPS'] == 'on' ? 'https' : 'http';
                $baseurl = \WHMCS\Utility\Environment\WebHelper::getBaseUrl();
                $url = $protocol.'://'.$_SERVER['HTTP_HOST'].$baseurl;
                $addFieldID = '<a target="_blank" href="'.$url.'/modules/registrars/EuroDNS/uploads/'.$addFieldID->first()->value.'?.'.time().'">VIEW</a>';
            }

            return [
                ED_IDT => '<select id="doc-type-select" class="form-control w-100" name="domainfield[101]" >
                    <option value="Australian Driver Licence">Australian Driver Licence</option>
                    <option value="Australian Passport">Australian Passport</option>
                    <option value="Australian Shooter or firearm licence">Australian Shooter or firearm licence</option>
                    <option value="Australian Security Licence">Australian Security Licence</option>
                    <option value="Australian Birth Certificate">Australian Birth Certificate</option>
                    <option value="Australian permanent resident visa">Australian permanent resident visa</option>
                    <option value="Australian Citizen Certificate">Australian Citizen Certificate</option>
                    <option value="Australian Certificate of Registration by Descent">Australian Certificate of Registration by Descent</option>
                </select>',
                ED_ID => '<input id="id-doc-file" type="file" name="document" class="form-control w-100" accept="image/gif,image/jpeg,image/png,image/webp,application/pdf"/>'.$addFieldID.'
                <script>

                jQuery("#doc-type-select option[value=\"'.$addFieldT.'\"]").prop("selected",true);

                jQuery("#id-doc-file").on("change",function(elem){
                    jQuery(this).closest("form").attr("enctype","multipart/form-data");
                });
                
                </script>',
            ];
        }
    }
});

add_hook('AdminClientDomainsTabFieldsSave', 1, function($vars) {

    if(array_key_exists('userid',$vars) && array_key_exists('id',$vars) && array_key_exists('registrar',$vars) && $vars['registrar'] === 'EuroDNS')
    {
        if(array_key_exists('domainfield',$vars) && array_key_exists(101,$vars['domainfield']))
        {
            $addField = GetAdditionalFieldFromDB($vars['id'],ED_IDT);
            
            if(!$addField)
            {
                InsertAdditionalFieldInDB($vars['id'],ED_IDT,$vars['domainfield'][101]);
            }
            else
            {
                UpdateAdditionalFieldInDB($vars['id'],ED_IDT,$vars['domainfield'][101]);
            }
        }


        if($_FILES && array_key_exists('document',$_FILES) && array_key_exists('tmp_name',$_FILES['document']) && array_key_exists('name',$_FILES['document']))
        {
            if ($_FILES['document']['error'] === UPLOAD_ERR_OK) {
                $file_path = dirname(__FILE__).'/uploads/';
                $fileName = $vars['userid'].'_'.$vars['id'].'.'.end(explode('.',$_FILES['document']['name']));
                
                if (!file_exists($file_path)) {
                    mkdir($file_path, 0777, true);
                }

                if(move_uploaded_file($_FILES['document']['tmp_name'],$file_path.$fileName))
                {
                    $addField = GetAdditionalFieldFromDB($vars['id'],ED_ID);

                    if(!$addField)
                    {
                        InsertAdditionalFieldInDB($vars['id'],ED_ID,$fileName);
                    }
                    else
                    {
                        UpdateAdditionalFieldInDB($vars['id'],ED_ID,$fileName);
                    }
                }else {
                    logModuleCall('EuroDNS', 'AdminClientDomainsTabFieldsSave', 'Failed to move uploaded file', compact('fileName', 'file_path'), null, null);
                }
            }
            else {
				logModuleCall('EuroDNS', 'AdminClientDomainsTabFieldsSave', 'Upload error code: ' . $_FILES['document']['error'], null, null, null);
			}
        }
    }
});